/*!
\file face.cpp
\brief Class definition for Face
\author Nicholas Phillips, phillips@friday.gsfc.nasa.gov
\date Feb 23, 2004
*/
#include <iostream>
#include <iomanip>
#include <fstream>
#include <chealpix>
#include "boundry.h"
#include "face.h"

using namespace std;

//================================================
//================================================
/*
long xy2pix(long ix, long iy) {
	static int x2pix[128];
	static int y2pix[128];
	static bool xy2pix_called = false;
	if( ! xy2pix_called ) {
		mk_xy2pix(x2pix, y2pix);
	xy2pix_called = true;
	}
	//int ix_low = (int)fmod(ix,128);
	int ix_low = ix % 128;
	int ix_hi  = ix / 128;
	int iy_low = iy % 128;
	int iy_hi  = iy / 128;
	return (x2pix[ix_hi]+y2pix[iy_hi]) * (128 * 128)
			+ (x2pix[ix_low]+y2pix[iy_low]);
}
*/
//=====================================================================================
//=====================================================================================
void Face::setRigging_NP(const int nside, vector<double> &costhetas)
{
	double ds = 1./nside;
	double s = 0, t = 0;
	Boundry lower;
	Boundry boundry0;
	Boundry boundry1;

	quadList.resize(nside);
	GLVertLI qli = quadList.begin();
	lower.set_eq( 0, 0.5*M_PI*(face+0.5), 1);
	for(int i = 0; i < nside; i++) {
		t = 0;
		qli->resize(2*(nside+1));
		GLVertI pti = qli->begin();
		bool boundry0_reset = false;
		bool boundry1_reset = false;
		boundry0.set_eq(costhetas[i],   lower(costhetas[i]),   -1);
		boundry1.set_eq(costhetas[i+1], lower(costhetas[i+1]), -1);
		for(int j = i; j <= i+nside; j++) {
			pti->setVert(acos(costhetas[j]),boundry0(costhetas[j]));
			pti->setTex(s,t);
			++pti;

			pti->setVert(acos(costhetas[j+1]),boundry1(costhetas[j+1]));
			pti->setTex(s+ds,t);
			++pti;

			if( (costhetas[j] >= 2./3.) && ! boundry0_reset) {
				boundry0.set_np(costhetas[j],boundry0(costhetas[j]),face);
				boundry0_reset = true;
			}

			if( (costhetas[j+1 ]>= 2./3.) && ! boundry1_reset) {
				boundry1.set_np(costhetas[j+1],boundry1(costhetas[j+1]),face);
				boundry1_reset = true;
			}

			t += ds;
		}
		s += ds;
		++qli;
	}
	return;
}
//=====================================================================================
void Face::setRigging_SP(const int nside, vector<double> &costhetas)
{
	double ds = 1./nside;
	double s = 0, t = 0;
	double phi_boundry = 0.5*M_PI*(face-7);
	Boundry boundry0;
	Boundry boundry1;

	quadList.resize(nside);
	GLVertLI qli = quadList.begin();
	for(int i = 0; i < nside; i++) {
		t = 0;
		qli->resize(2*(nside+1));
		GLVertI pti = qli->begin();
		bool boundry0_reset = false;
		bool boundry1_reset = false;
		boundry0.set_sp(costhetas[i],   phi_boundry, face);
		boundry1.set_sp(costhetas[i+1], phi_boundry, face);
		for(int j = i; j <= i+nside; j++) {
			pti->setVert(acos(costhetas[j]),boundry0(costhetas[j]));
			pti->setTex(s,t);
			++pti;

			pti->setVert(acos(costhetas[j+1]),boundry1(costhetas[j+1]));
			pti->setTex(s+ds,t);
			++pti;

			if( (costhetas[j] > (-2./3. - 0.001)) && ! boundry0_reset) {
				boundry0.set_eq(costhetas[j],boundry0(costhetas[j]),-1);
				boundry0_reset = true;
			}
			if( (costhetas[j+1] > (-2./3. - 0.001)) && ! boundry1_reset) {
				boundry1.set_eq(costhetas[j+1],boundry1(costhetas[j+1]),-1);
				boundry1_reset = true;
			}

			t += ds;
		}
		s += ds;
		++qli;
	}
	return;
}
//=====================================================================================
void Face::setRigging_EQ(const int nside, vector<double> &costhetas)
{
	double ds = 1./nside;
	double s = 0, t = 0;
	Boundry lower;
	lower.set_eq(-2./3., 0.5*M_PI*(face-4), 1);
	Boundry boundry0;
	Boundry boundry1;

	quadList.resize(nside);
	GLVertLI qli = quadList.begin();
	for(int i = 0; i < nside; i++) {
		t = 0;
		qli->resize(2*(nside+1));
		GLVertI pti = qli->begin();
		boundry0.set_eq(costhetas[i],   lower(costhetas[i]),   -1);
		boundry1.set_eq(costhetas[i+1], lower(costhetas[i+1]), -1);
		for(int j = i; j <= i+nside; j++) {
			pti->setVert(acos(costhetas[j]),boundry0(costhetas[j]));
			pti->setTex(s,t);
			++pti;

			pti->setVert(acos(costhetas[j+1]),boundry1(costhetas[j+1]));
			pti->setTex(s+ds,t);
			++pti;

			t += ds;
		}
		s += ds;
		++qli;
	}
	return;
}
//=====================================================================================
void Face::setRigging(const int nside, vector<double> &costhetas)
{
	if( face < 4)
		setRigging_NP(nside,costhetas);
	else if( face > 7)
		setRigging_SP(nside,costhetas);
	else
		setRigging_EQ(nside,costhetas);
	rigging_set = true;

	GLVertLI qli = quadList.begin();
	for(int i = 0; i < nside; i++) {
		GLVertI pti = qli->begin();
		GLVertI pti_end = qli->end();
		for(; pti != pti_end; ++pti) {
			pti->s = 0.25*pti->s + 0.25*(face % 4);
			pti->t = 0.25*pti->t + 0.25*(face / 4);
		}
		++qli;
	}

	//toMollweide();
	
	return;
}
//=====================================================================================
void Face::draw()
{
	if( ! rigging_set ) {
		cerr << "Face::draw:Rigging not set, face = " << face << endl;
		return;
	}

	//glColor3f(0,0,0);
	//nglEnable(GL_TEXTURE_2D );
	GLVertLI qli = quadList.begin();
	GLVertLI qli_end = quadList.end();
	GLVertI pti;
	GLVertI pti_end;
	for(; qli !=  qli_end; ++qli) {
		pti_end = qli->end();
		glBegin(GL_QUAD_STRIP);
		//glBegin(GL_LINE_STRIP);
		for(pti = qli->begin(); pti != pti_end; ++pti) {
			pti->glSet();
		}
		glEnd();
	}
	return;
}

