/*!
\file mollweide.cpp
\brief Class definition for Mollweide
\author Nicholas Phillips, phillips@friday.gsfc.nasa.gov
\date Feb 23, 2004
*/
#include <iostream>
#include "mollweide.h"

using namespace std;

//=====================================================================================
double toMollweide(const double phi, const double lambda, double &x, double &y)
{
	x = phi;
	y = lambda;

	const double eps = 1e-6; 	// how well we solve for theta
	const int nmax = 20;		// max number of iterations for Newton's method
	const double lambda0 = 0;	// offset
	// Need to solve 2theta + sin(2theta) == pi sin(phi), for theta
	double theta;
	// Upper limit
	if( fabs(phi - M_PI/2) < 1e-6) {
		theta = M_PI/2;
	}
	// Lower limit
	else if(fabs(phi + M_PI/2) < 1e-6 ) {
		theta = -M_PI/2;
	}
	else {
		double dtheta;
		double f,df;
		// Use Newton's method to solve
		// Initial guess
		theta = phi/2;
		// function for whose zero we solve for
		f = 2*theta + sin(2*theta) - M_PI*sin(phi);
		int i;
		for(i = 0; i < nmax; i++) {
			// are we close enough to zero?
			if( fabs(f) < eps)
				break;
			// derivation of f
			df = 2*(1+cos(2*theta));
			// Newton's method error
			dtheta = -f/df;
			// update guess
			theta += dtheta;
			// new value of function
			f = 2*theta + sin(2*theta) - M_PI*sin(phi);
		}
	}
	// We now have the theta we need
	x = 2*sqrt(2.)*(lambda- lambda0)*cos(theta)/M_PI;
	y = sqrt(2.)*sin(theta);
	return theta;
}

//=====================================================================================
void Face::toMollweide()
{
	double r = 1/sqrt(2.);
	double bump = 0.01;
	double phi;
	double lambda;
	double x,y;

	GLVertLI qli = quadList.begin();
	GLVertLI qli_end = quadList.end();
	GLVertI pti;
	GLVertI pti_end;
	for(; qli !=  qli_end; ++qli) {
		pti_end = qli->end();
		for(pti = qli->begin(); pti != pti_end; ++pti) {
			phi = asin(pti->z);
			lambda = atan2(pti->y,pti->x);
			::toMollweide(phi,  lambda, x, y);
			if( (face == 2 || face == 10) && x > 0) x *= -1;
			pti->y = r*x;
			pti->z = r*y;
			pti->x = -bump*(pti->z*pti->z + pti->y*pti->y);
		}
	}
	return;
}

//=====================================================================================
void Mollweide::setResolution(int sphere_res_)
{
	Sphere3D::setResolution(sphere_res_);
	for(int i = 0; i < 12; i++)
		faces[i].toMollweide();

	return;
}

//=====================================================================================
bool Mollweide::select(const qglviewer::Vec &o,const qglviewer::Vec &d, qglviewer::Vec &v)
{
	return false;
}
