/*!
\file skyviewer.h
\brief Class declaration for  SkyViewer
\author Nicholas Phillips, phillips@friday.gsfc.nasa.gov
\date Oct  6, 2004
*/
#ifndef SKYVIEWER_H
#define SKYVIEWER_H


#include <vector>
#include <map>
#include <QGLViewer/qglviewer.h>
#include "typedefs.h"
#include "skymap.h"
#include "skytexture.h"
#include "projection.h"
#include "sphere3d.h"
#include "mollweide.h"
#include "selector.h"

class SkyViewer : public QGLViewer
{
    Q_OBJECT
public :
	SkyViewer(QWidget *parent,const char *name);
	//SkyViewer::~SkyViewer();
	virtual ~SkyViewer();
signals:
	//! Send a string indicating the current max value
	void tellValueMax(const QString &);
	//! Send an integer indicating the current max value
	void tellValueMax(int);
	//! Send out the info for the selected pixel
	void tellPixelInfo(const QString &);

public slots:
	//! Load a FITS file
	void loadFile();
	//! Re-Load the current FITS file
	void reloadFile();
	//! Change to the selected view mode
	void setDisplayMode(const QString &);
	//! Change to display the selected quantity
	void setDisplayValue(const QString &);
	//! Change the max and min values
	void setValueMax(int);
	//! Note the current incoming string
	void setValueMax(const QString &);
	//! Time to try and turn \c maxstr in a value
	void setValueMax();
	//! Time to try and turn \c maxstr in a value
	void setValueMax2();
	//! Coarseness of the spherical mesh
	void setSphereRes(int);
	//! So, how do I use this?
	void showHelp();

protected:
	virtual void init();
	virtual void draw();
	virtual void animate();
	virtual void keyPressEvent(QKeyEvent *e);
	virtual QString helpString() const;
	virtual void postSelection(int x, int y);

private:
	//! What is the current max for the current quantity
	float getmax();
	//! Fill the skycolor vector based on the loaded skymap and current state
	void setSkyColor();
	//! Set the OpenGL texture
	void setTexture();
private :
	//! List of types of projections
	enum display_modes { sphere, mollweide, stereo };

	QString currentfname;
	Skymap sky;		//!< The loaded FITS map
	Pixel max0;		//!< The abs max values in the current map
	Pixel max;		//!< Max values in current use
	QString maxstr;	//!< Incoming max value string
	display_modes current_mode;	//!< Received signals should change this
	DisplayValue current_value;//!< Received signals should change this

	Projection *projection;		//!< Current Projection
	Sphere3D sphere3d_proj;		//!< Spherical Projection
	Mollweide mollweide_proj;	//!< Mollweide Projection


	SkyTexture texture;

	int tex_res;									//!< The resolution of the current texture
	int sphere_res;									//!< The current resolution of the 3D sphere

	ColorTable skycolor;		//!< qRGB of each pixel in the skymap
	ColorTable colors;			//!< The current color table
	int ncolors;				//!< Size of the current colar table
	bool firstcall;				//!< Is this the first time I am being drawn?
	QString cwd;				//!< current working directory
	qglviewer::CameraConstraint *constraint;

	Selector selector;			//!< Control selecting pixels
//	float sc;
//	Throb throb;
};

#endif
